import 'dart:io';

import 'package:flutter/material.dart';
import 'package:nextcloud_reminder/addReminder.dart';
import 'package:nextcloud_reminder/parser/todotxt.dart';
import 'package:nextcloud_reminder/repeating_task.dart';
import 'package:nextcloud_reminder/table.dart';
import 'package:nextcloud_reminder/types/tasks.dart';
import 'package:path_provider/path_provider.dart';

class HomeWidget extends StatefulWidget {
  const HomeWidget({super.key, required this.title});

  // This widget is the home page of your application. It is stateful, meaning
  // that it has a State object (defined below) that contains fields that affect
  // how it looks.

  // This class is the configuration for the state. It holds the values (in this
  // case the title) provided by the parent (in this case the App widget) and
  // used by the build method of the State. Fields in a Widget subclass are
  // always marked "final".

  final String title;

  @override
  State<HomeWidget> createState() => _HomeWidgetState();
}

class _HomeWidgetState extends State<HomeWidget> with WidgetsBindingObserver {

  final ScrollTable _checkboxes = ScrollTable(title: "TODOs");
  int _counter = 1;
  late final Directory appDocDirectory;
  late final File todotxt = File('${appDocDirectory.path}/todo.txt');
  final List<Task> tasks = [];

  void _loadTodos() {
    for (var element in tasks) {
      debugPrint("loaded: ${element.toString()}");
      _checkboxes.addTask(RepeatingTask(task: element));
    }
  }

  void initLazy() async {
    appDocDirectory = (await getApplicationDocumentsDirectory());
    if (await todotxt.exists()) {
      var data = await todotxt.readAsLines();
      debugPrint(data.toString());
      tasks.addAll(TodoParser.parse(data));
      _loadTodos();
    } else {
      debugPrint("no todo file found at ${todotxt.path}");
    }
  }

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addObserver(this);
    initLazy();
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused || state == AppLifecycleState.inactive) {
      // App is in the background, save any unsaved data
      debugPrint("save to disk");
      saveData();
    }
  }
  void saveData() async {
    //TODO: better update lines instead of blindly overwriting.
    String data = tasks.map((t) => t.toString()).join("\n");
    debugPrint("Saving:\n$data");
    await todotxt.writeAsString(data);
  }

  void _addDummyTask() {
    setState(() {
      // This call to setState tells the Flutter framework that something has
      // changed in this State, which causes it to rerun the build method below
      // so that the display can reflect the updated values. If we changed
      // stuff without calling setState(), then the build method would not be
      // called again, and so nothing would appear to happen.
      _checkboxes.addTask(RepeatingTask(task: Task(title: "Dummy Task #$_counter", begin: DateTime.now()), repeat: _counter++));
    });
  }
  void _addTask(RepeatingTask? t) {
    if (t != null) {
      setState(() {
        _checkboxes.addTask(t!);
        tasks.add(t.task);
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    // This method is rerun every time setState is called, for instance as done
    // by the _incrementCounter method above.
    //
    // The Flutter framework has been optimized to make rerunning build methods
    // fast, so that you can just rebuild anything that needs updating rather
    // than having to individually change instances of widgets.
    return Scaffold(
      appBar: AppBar(
        // Here we take the value from the MyHomePage object that was created by
        // the App.build method, and use it to set our appbar title.
        title: Text(widget.title),
      ),
      body: Center(
        // Center is a layout widget. It takes a single child and positions it
        // in the middle of the parent.
        child: _checkboxes,
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: () => Navigator.of(context).push(
          MaterialPageRoute(builder: (context) => AddTaskWidget(onSave: _addTask))
        ),
        tooltip: 'add Task',
        child: const Icon(Icons.add),
      ), // This trailing comma makes auto-formatting nicer for build methods.
    );
  }
}
